///////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  BocaBTController.m
//  BocaLibTester
//
//  Created by Michael Hall on 9/13/16.
//  Copyright © 2016 Boca Systems. All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>
#import "BocaBTController.h"

@class BocaStoryBoardAppMainViewController;

@implementation EAController

@synthesize protocolString = _protocolString;
@synthesize writeDataBuffer = _writeDataBuffer;
@synthesize accessoryVC = _accessoryVC;
extern NSString *s;
NSString *SerialName;

#pragma mark Internal

BOOL Open_BT()
{
    BOOL Status;
    SerialName = @"";
    if([[EAController sharedController] openSession])
    {
        Status = true;                //return positive status
    }
    else  //Ipad must have established a pair Blue Tooth connection before this application can find it
    {
        Status = false;               //return negative status
    }
    return(Status);
    
}

BOOL Open_BTPicker(NSString *name)
{
    BOOL Status;
    SerialName = name;
    if([[EAController sharedController] openSession])
    {
        Status = true;                //return positive status
    }
    else  //Ipad must have established a pair Blue Tooth connection before this application can find it
    {
        Status = false;               //return negative status
    }
    return(Status);
    
}

void Close_BT()
{
    [[EAController sharedController] closeSession];
}

void Write_BT(NSString *string)
{
    NSData* command=[string dataUsingEncoding:NSUTF8StringEncoding];
    
    const uint8_t *buf = (const uint8_t*)[command bytes];
    CFIndex bufLen = command.length;
    [[EAController sharedController]writeData:[NSData dataWithBytes:buf length:bufLen]];
}

void Write_BT_Data(uint8_t *buf, int bufLen)
{
    [[EAController sharedController]writeData:[NSData dataWithBytes:buf length:bufLen]];
}


NSString *Read_BT()
{
    return(s);
}

void ClearStatus_BT()
{
    s = @"";
}


- (void)writeDataFromBufferToStream
{
    //NSLog(@"EAController::writeDataFromBufferToStream");
    while (([[_session outputStream] hasSpaceAvailable]) && ([_writeDataBuffer length] > 0))
    {
        NSInteger bytesWritten = [[_session outputStream] write:[_writeDataBuffer bytes] maxLength:[_writeDataBuffer length]];
        
        //NSLog(@"[%lu] bytes in buffer - wrote [%d] bytes", (unsigned long)[_writeDataBuffer length], bytesWritten);
        //NSLog(@"Data written = %@ ",_writeDataBuffer);
        //NSLog(@"[%lu] bytes in buffer - wrote [%ld] bytes", (unsigned long)[_writeDataBuffer length], (long)bytesWritten);
        if (bytesWritten == -1)
        {
            //NSLog(@"write error");
            break;
        }
        else if (bytesWritten > 0)
        {
            [_writeDataBuffer replaceBytesInRange:NSMakeRange(0, bytesWritten) withBytes:NULL length:0];
        }
    }
}


#define EAD_INPUT_BUFFER_SIZE 128
- (void)readDataFromStreamToBuffer
{
    //NSLog(@"EAController::readDataFromStreamToBuffer");
    uint bytesAppended=0;
    uint bytesProcessed=0;
    uint8_t buf[EAD_INPUT_BUFFER_SIZE];
    NSString * Status=@"";
    while ([[_session inputStream] hasBytesAvailable])
    {
        NSInteger bytesRead = [[_session inputStream] read:buf maxLength:EAD_INPUT_BUFFER_SIZE];
        //NSLog(@"read %ld bytes from input stream", (long)bytesRead);
        
        while(bytesProcessed < bytesRead)
        {
            if(buf[bytesAppended] < 32)                      //if non-printable character, look at as status
            {
                switch (buf[bytesAppended])                  //check for most common status
                {
                    case 0x6:
                        Status = @"Ticket ACK";
                        break;
                    case 0x8:
                        Status = @"Invalid Checksum";
                        break;
                    case 0x9:
                        Status = @"Valid Checksum";
                        break;
                    case 0x10:
                        Status = @"Out of Tickets";
                        break;
                    case 0x11:
                        Status = @"X-On";
                        break;
                    case 0x12:
                        Status = @"Power On";
                        break;
                    case 0x13:
                        Status = @"X-Off";
                        break;
                    case 0x15:
                        Status = @"Ticket NAK";
                        break;
                    case 0x18:
                        Status = @"Ticket Jam";
                        break;
                    case 0x1D:
                        Status = @"Cutter Jam";
                        break;
                    default:
                        Status = @"";
                        //Status = [NSString stringWithFormat:@"%@%d",@"Non Standard Status Value = ",buf[bytesAppended]];
                        break;
                }
                bytesAppended++;
                bytesProcessed++;
                
                //NSLog(@"A Status = %@",Status);
                
                
            }
            else                                             //printable character
            {
                Status = [[NSString alloc] initWithBytes:buf length:(NSUInteger)bytesRead encoding:NSUTF8StringEncoding];
                //NSLog(@"B Status = %@",Status);
                
                bytesProcessed += bytesRead;                 //@mdh
                bytesAppended += bytesRead;                  //@mdh
            }
            
            //Concatenate return status into buffer, waiting to return to parent application
            s = [NSString stringWithFormat:@"%@%@\n",s,Status];

        }
    }
}

#pragma mark Public Methods

+ (EAController *)sharedController
{
    static EAController *_eaController = nil;
    if (_eaController == nil)
    {
        _eaController = [[EAController alloc] init];
    }
    
    return _eaController;
}



- (id)init
{
    
    //NSLog(@"EAController::init");
    
    //iPads can be paired to mulitiple Boca BlueTooth LE printers at the same time from
    //the main iPad screen using "Settings" -> Bluetooth
    //I tested with 3 Boca BT LE printers and by manipulating the selected BT index value 0 to 2
    //I was able to open/close RW channels with each, one at a time
    
    //int selectedBTindex = 0;
    int found = -1;
    int x = 0;

    if (self = [super init])
    {
        //initialization
        _writeDataBuffer = [[NSMutableData alloc] init];
        _accessoryList = [[NSMutableArray alloc] initWithArray:[[EAAccessoryManager sharedAccessoryManager] connectedAccessories]];
        
        //Debug - display number of connected BT devices
        //NSLog(@"accessory count A: %lu", (unsigned long)[_accessoryList count]);
        if ([_accessoryList count])
        {

        
            //This code can be reached through Open_BT() or Open_BTPicker(NSString *name).  If reached through OpenBT() then
            //the SerialName field will be blank, so this routine will open the default/only printer currently paired to the iPad.
            //Else if a name was provided through a call from Open_BTPicker(NSString *name) then parse the accessory list below
            //to find the matching SerialName and connect to it based on its index number
            if([SerialName isEqual:@""])
            {
                //display number of connected BT devices on GUI status screen
                //NSString *connectedcountstr = [NSString stringWithFormat:@"%@%d",@"Connected Accessory Count ",[_accessoryList count]];
                //[BocaStoryBoardAppMainViewControllerobject DisplayText:connectedcountstr];
            
                //Use the default/only paired printer using index zero
                //_selectedAccessory = [_accessoryList objectAtIndex:selectedBTindex];
                _selectedAccessory = [_accessoryList objectAtIndex:0];
                NSArray *protocolStrings = [_selectedAccessory protocolStrings];
                self.protocolString = [protocolStrings objectAtIndex:0];
            
            }
            else //if ([_accessoryList count] > 1)
            {
                //Find the printer by SerialName
                for (x = 0; x < [_accessoryList count]; x++)
                {
                    _selectedAccessory = [_accessoryList objectAtIndex:x];
                    NSString *name = [_selectedAccessory name];

                    if([name isEqualToString:SerialName])
                    {
                        found = x;
                        x = (int)([_accessoryList count] + 1);
                    }
                }
            
//              _selectedAccessory = [_accessoryList objectAtIndex:selectedBTindex];
                if(found >= 0)
                {
                    NSArray *protocolStrings = [_selectedAccessory protocolStrings];
                    self.protocolString = [protocolStrings objectAtIndex:0];
                }
                else
                {
                    //Use the default/only paired printer using index zero
                    _selectedAccessory = [_accessoryList objectAtIndex:0];
                    NSArray *protocolStrings = [_selectedAccessory protocolStrings];
                    self.protocolString = [protocolStrings objectAtIndex:0];
                }
            }
        }
    }
    return self;
}

- (EAAccessory *)selectedAccessory
{
    int found = -1;
    int x = 0;
    //int selectedBTindex = 0;
    
    //NSLog(@"selectedAccessory");
    if (_selectedAccessory == nil)
    {
        _accessoryList = [[NSMutableArray alloc] initWithArray:[[EAAccessoryManager sharedAccessoryManager] connectedAccessories]];
        
        //Debug - display number of connected BT devices
        //NSLog(@"accessory count B: %lu", (unsigned long)[_accessoryList count]);
        if ([_accessoryList count])
        {
            
            
            //This code can be reached through Open_BT() or Open_BTPicker(NSString *name).  If reached through OpenBT() then
            //the SerialName field will be blank, so this routine will open the default/only printer currently paired to the iPad.
            //Else if a name was provided through a call from Open_BTPicker(NSString *name) then parse the accessory list below
            //to find the matching SerialName and connect to it based on its index number
            if([SerialName isEqual:@""])
            {
                //display number of connected BT devices on GUI status screen
                //NSString *connectedcountstr = [NSString stringWithFormat:@"%@%d",@"Connected Accessory Count ",[_accessoryList count]];
                //[BocaStoryBoardAppMainViewControllerobject DisplayText:connectedcountstr];
                
                //Use the default/only paired printer using index zero
                //_selectedAccessory = [_accessoryList objectAtIndex:selectedBTindex];
                _selectedAccessory = [_accessoryList objectAtIndex:0];
                NSArray *protocolStrings = [_selectedAccessory protocolStrings];
                self.protocolString = [protocolStrings objectAtIndex:0];
                
            }
            else //if ([_accessoryList count] > 1)
            {
                //Find the printer by SerialName
                for (x = 0; x < [_accessoryList count]; x++)
                {
                    _selectedAccessory = [_accessoryList objectAtIndex:x];
                    NSString *name = [_selectedAccessory name];
                    
                    if([name isEqualToString:SerialName])
                    {
                        found = x;
                        x = (int)([_accessoryList count] + 1);
                    }
                }
                
                if(found >= 0)
                {
                    NSArray *protocolStrings = [_selectedAccessory protocolStrings];
                    self.protocolString = [protocolStrings objectAtIndex:0];
                }
                else
                {
                    //Use the default/only paired printer using index zero
                    _selectedAccessory = [_accessoryList objectAtIndex:0];
                    NSArray *protocolStrings = [_selectedAccessory protocolStrings];
                    self.protocolString = [protocolStrings objectAtIndex:0];
                }
            }
        }
    }
    return _selectedAccessory;
}


- (void)dealloc
{
    [self closeSession];
    [self setupControllerForAccessory:nil withProtocolString:nil];
    
    _writeDataBuffer = nil;
    _accessoryVC = nil;
}


- (void)setupControllerForAccessory:(EAAccessory *)accessory withProtocolString:(NSString *)protocolString
{
    //NSLog(@"EAController::setupControllerForAccessory:");
    _selectedAccessory = [accessory copy];
    _protocolString = [protocolString copy];
}

//Pick default BT device
- (BOOL)openSession
{
    if (_session == nil)
    {
        //NSLog(@"EAController::openSession");
        [_selectedAccessory setDelegate:self];
        
        _session = [[EASession alloc] initWithAccessory:[self selectedAccessory] forProtocol:_protocolString];
        
        if (_session)
        {
            [[_session inputStream] setDelegate:self];
            [[_session inputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
            [[_session inputStream] open];
            
            [[_session outputStream] setDelegate:self];
            [[_session outputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
            [[_session outputStream] open];
            //NSLog(@"opened the session");
            //enable printer function buttons
            
        }
        //else
        //{
        //NSLog(@"creating session failed");
        //}
    }
    
    return (_session != nil);
}

//This routine allows selection of BT device based on name
- (BOOL)openSessionPicker
{
    __block BOOL Status = false;
    if (_session == nil)
    {
        //NSLog(@"EAController::openSessionPicker");
        [_selectedAccessory setDelegate:self];
        
        [[EAAccessoryManager sharedAccessoryManager] showBluetoothAccessoryPickerWithNameFilter:nil completion:^(NSError *error)
        {
            if (!error)
            {
                Status = true;
                //NSLog(@"You made it! Well done!!!");
            }
            //else
            //{
                //NSLog(@"error :%@", error);
            //}
        }];
    }
    return(Status);
}


- (void)closeSession
{
    //NSLog(@"EAController::closeSession");
    [[_session inputStream] close];
    [[_session inputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [[_session inputStream] setDelegate:nil];
    [[_session outputStream] close];
    [[_session outputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [[_session outputStream] setDelegate:nil];
    
    //@mdh BT [_session release];
    _session = nil;
    _selectedAccessory = nil;           //Added this for version 2.0  needed to properly end a session so another can be opened
    //NSLog(@"closed the session");
}


- (void)writeData:(NSData *)data
{
    //NSLog(@"EAController::writeData:");
    [_writeDataBuffer appendData:data];
    [self writeDataFromBufferToStream];
}


- (BOOL)isAccessoryConnected
{
    //NSLog(@"EAController::isAccessoryConnected");
    if (_selectedAccessory && [_selectedAccessory isConnected])
        return YES;
    else
        return NO;
}


#pragma mark -
#pragma mark EAAccessoryDelegate

- (void)accessoryDidDisconnect:(EAAccessory *)accessory
{
    //NSLog(@"EAController::accessoryDidDisconnect:");
    // do something ...
    //if (![self isAccessoryConnected])
    //{
    
    //}
}


#pragma mark -
#pragma mark NSStreamDelegateEventExtensions

- (void)stream:(NSStream *)aStream handleEvent:(NSStreamEvent)eventCode
{
    //NSLog(@"EAController::handleEvent:");
    switch (eventCode) {
        case NSStreamEventNone:
            //NSLog(@"stream %@ event none", aStream);
            break;
        case NSStreamEventOpenCompleted:
            //NSLog(@"stream %@ event open completed", aStream);
            break;
        case NSStreamEventHasBytesAvailable:
            //NSLog(@"stream %@ event bytes available", aStream);
            [self readDataFromStreamToBuffer];
            break;
        case NSStreamEventHasSpaceAvailable:
            //NSLog(@"stream %@ event space available", aStream);
            [self writeDataFromBufferToStream];
            break;
        case NSStreamEventErrorOccurred:
            //NSLog(@"stream %@ event error", aStream);
            break;
        case NSStreamEventEndEncountered:
            //NSLog(@"stream %@ event end encountered", aStream);
            break;
        default:
            break;
    }
}


#pragma mark -
#pragma mark NSNotifications

- (void)accessoryConnected:(NSNotification *)notification
{
    //NSLog(@"EAController::accessoryConnected");
    if (![self isAccessoryConnected])
    {
        EAAccessory *a = [self selectedAccessory];
        [self setupControllerForAccessory:a withProtocolString:_protocolString];
        [self closeSession];
        [self openSession];
    }
}


- (void)accessoryDisconnected:(NSNotification *)notification
{
    //NSLog(@"EAController::accessoryDisconnected");
    if (![self isAccessoryConnected])
    {
        EAAccessory *a = [self selectedAccessory];
        [self setupControllerForAccessory:a withProtocolString:_protocolString];
        [self closeSession];
        [self openSession];
    }
}

@end
