///////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  BocaUSBController.m
//  BocaIpadSDK
//
//  Created by Michael Hall on 11/16/17.
//  Copyright © 2017 Boca Systems. All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>
#import "BocaUSBController.h"

@class BocaStoryBoardAppMainViewController;

@implementation EAControllerUSB

@synthesize protocolString = _protocolString;
@synthesize writeDataBuffer = _writeDataBuffer;
@synthesize accessoryVC = _accessoryVC;
extern NSString *s;

#pragma mark Internal


BOOL Open_USB()
{
    BOOL Status;
    //NSString *Test=@"com.Shure.usbdemo";

    //- (EASession *)openSessionForProtocol:(NSString *)protocolString
    //@"com.Shure.usbdemo"
    //    if([[EAControllerUSB sharedController] openSession])

    //if([[EAControllerUSB sharedController] openSessionForProtocol:Test])
    if([[EAControllerUSB sharedController] openSession])
    {
        Status = true;                //return positive status
    }
    else  //Ipad must have established a USB connection
    {
        Status = false;               //return negative status
    }
    return(Status);
    
}


void Close_USB()
{
    [[EAControllerUSB sharedController] closeSession];
}

void Write_USB(NSString *string)
{
    NSData* command=[string dataUsingEncoding:NSUTF8StringEncoding];
    
    const uint8_t *buf = (const uint8_t*)[command bytes];
    CFIndex bufLen = command.length;
    [[EAControllerUSB sharedController]writeData:[NSData dataWithBytes:buf length:bufLen]];
}

void Write_USB_Data(uint8_t *buf, int bufLen)
{
    [[EAControllerUSB sharedController]writeData:[NSData dataWithBytes:buf length:bufLen]];
}


NSString *Read_USB()
{
    return(s);
}

void ClearStatus_USB()
{
    s = @"";
}


- (void)writeDataFromBufferToStream
{
    //NSLog(@"EAController::writeDataFromBufferToStream");
    while (([[_session outputStream] hasSpaceAvailable]) && ([_writeDataBuffer length] > 0))
    {
        NSInteger bytesWritten = [[_session outputStream] write:[_writeDataBuffer bytes] maxLength:[_writeDataBuffer length]];
        
        //NSLog(@"[%lu] bytes in buffer - wrote [%d] bytes", (unsigned long)[_writeDataBuffer length], bytesWritten);
        //NSLog(@"Data written = %@ ",_writeDataBuffer);
        //NSLog(@"[%lu] bytes in buffer - wrote [%ld] bytes", (unsigned long)[_writeDataBuffer length], (long)bytesWritten);
        if (bytesWritten == -1)
        {
            //NSLog(@"write error");
            break;
        }
        else if (bytesWritten > 0)
        {
            [_writeDataBuffer replaceBytesInRange:NSMakeRange(0, bytesWritten) withBytes:NULL length:0];
        }
    }
}


#define EAD_INPUT_BUFFER_SIZE 128
- (void)readDataFromStreamToBuffer
{
    //NSLog(@"EAController::readDataFromStreamToBuffer");
    uint bytesAppended=0;
    uint bytesProcessed=0;
    uint8_t buf[EAD_INPUT_BUFFER_SIZE];
    NSString * Status=@"";
    while ([[_session inputStream] hasBytesAvailable])
    {
        NSInteger bytesRead = [[_session inputStream] read:buf maxLength:EAD_INPUT_BUFFER_SIZE];
        //NSLog(@"read %ld bytes from input stream", (long)bytesRead);
        
        while(bytesProcessed < bytesRead)
        {
            if(buf[bytesAppended] < 32)                      //if non-printable character, look at as status
            {
                switch (buf[bytesAppended])                  //check for most common status
                {
                    case 0x6:
                        Status = @"Ticket ACK";
                        break;
                    case 0x8:
                        Status = @"Invalid Checksum";
                        break;
                    case 0x9:
                        Status = @"Valid Checksum";
                        break;
                    case 0x10:
                        Status = @"Out of Tickets";
                        break;
                    case 0x11:
                        Status = @"X-On";
                        break;
                    case 0x12:
                        Status = @"Power On";
                        break;
                    case 0x13:
                        Status = @"X-Off";
                        break;
                    case 0x15:
                        Status = @"Ticket NAK";
                        break;
                    case 0x18:
                        Status = @"Ticket Jam";
                        break;
                    case 0x1D:
                        Status = @"Cutter Jam";
                        break;
                    default:
                        Status = @"";
                        //Status = [NSString stringWithFormat:@"%@%d",@"Non Standard Status Value = ",buf[bytesAppended]];
                        break;
                }
                bytesAppended++;
                bytesProcessed++;
                
                //NSLog(@"A Status = %@",Status);
                
                
            }
            else                                             //printable character
            {
                Status = [[NSString alloc] initWithBytes:buf length:(NSUInteger)bytesRead encoding:NSUTF8StringEncoding];
                //NSLog(@"B Status = %@",Status);
                
                bytesProcessed += bytesRead;                 //@mdh
                bytesAppended += bytesRead;                  //@mdh
            }
            
            //Concatenate return status into buffer, waiting to return to parent application
            s = [NSString stringWithFormat:@"%@%@\n",s,Status];
            
        }
    }
}



#pragma mark Public Methods

+ (EAControllerUSB *)sharedController
{
    static EAControllerUSB *_eaController = nil;
    if (_eaController == nil)
    {
        _eaController = [[EAControllerUSB alloc] init];
    }
    
    return _eaController;
}



- (id)init
{
    
    //NSLog(@"EAControllerUSB::init");
    
    //iPads can be connected to mulitiple Boca BlueTooth LE printers at the same time from
    //the main iPad screen using "Settings" -> Bluetooth
    //I tested with 3 Boca BT LE printers and by manipulating the selected BT index value 0 to 2
    //I was able to connect/disconnect with each.  If someone is interested in that approach
    //add something to the main GUI so that the user can indicate which Boca BT printer they
    //want to use when multiples are available.  For now the first printer found (with index 0)
    //will be selected.  Same applies to selectedAccessory function below
    int selectedBTindex = 0;
    
    if (self = [super init])
    {
        //initialization
        _writeDataBuffer = [[NSMutableData alloc] init];
        _accessoryList = [[NSMutableArray alloc] initWithArray:[[EAAccessoryManager sharedAccessoryManager] connectedAccessories]];
        //NSLog(@"accessory count A: %lu", (unsigned long)[_accessoryList count]);
        if ([_accessoryList count])
        {
            //display number of connected BT devices on GUI status screen
            //NSString *connectedcountstr = [NSString stringWithFormat:@"%@%d",@"Connected Accessory Count ",[_accessoryList count]];
            //[BocaStoryBoardAppMainViewControllerobject DisplayText:connectedcountstr];
            
            _selectedAccessory = [_accessoryList objectAtIndex:selectedBTindex];
            NSArray *protocolStrings = [_selectedAccessory protocolStrings];
            self.protocolString = [protocolStrings objectAtIndex:0];
            
        }
    }
    return self;
}


- (EAAccessory *)selectedAccessory
{
    int selectedBTindex = 0;
    
    //NSLog(@"selectedAccessory");
    if (_selectedAccessory == nil)
    {
        _accessoryList = [[NSMutableArray alloc] initWithArray:[[EAAccessoryManager sharedAccessoryManager] connectedAccessories]];
        //NSLog(@"accessory count B: %lu", (unsigned long)[_accessoryList count]);
        if ([_accessoryList count])
        {
            _selectedAccessory = [_accessoryList objectAtIndex:selectedBTindex];
            NSArray *protocolStrings = [_selectedAccessory protocolStrings];
            self.protocolString = [protocolStrings objectAtIndex:0];
        }
    }
    return _selectedAccessory;
}


- (void)dealloc
{
    [self closeSession];
    [self setupControllerForAccessory:nil withProtocolString:nil];
    
    _writeDataBuffer = nil;
    _accessoryVC = nil;
}


- (void)setupControllerForAccessory:(EAAccessory *)accessory withProtocolString:(NSString *)protocolString
{
    //NSLog(@"EAControllerUSB::setupControllerForAccessory:");
    _selectedAccessory = [accessory copy];
    _protocolString = [protocolString copy];
}


- (BOOL)openSession
{
    if (_session == nil)
    {
        //NSLog(@"EAControllerUSB::openSession");
        [_selectedAccessory setDelegate:self];
        
        _session = [[EASession alloc] initWithAccessory:[self selectedAccessory] forProtocol:@"com.Shure.usbdemo"];
        //_session = [[EASession alloc] initWithAccessory:[self selectedAccessory] forProtocol:_protocolString];
        
        if (_session)
        {
            [[_session inputStream] setDelegate:self];
            [[_session inputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
            [[_session inputStream] open];
            
            [[_session outputStream] setDelegate:self];
            [[_session outputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
            [[_session outputStream] open];
            //NSLog(@"opened the session");
            //enable printer function buttons
            
        }
        //else
        //{
        //NSLog(@"creating session failed");
        //}
    }
    
    return (_session != nil);
}


- (void)closeSession
{
    //NSLog(@"EAControllerUSB::closeSession");
    [[_session inputStream] close];
    [[_session inputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [[_session inputStream] setDelegate:nil];
    [[_session outputStream] close];
    [[_session outputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [[_session outputStream] setDelegate:nil];
    
    //@mdh BT [_session release];
    _session = nil;
    _selectedAccessory = nil;           //Added this for version 2.0  needed to properly end a session so another can be opened
    //NSLog(@"closed the session");
}


- (void)writeData:(NSData *)data
{
    //NSLog(@"EAControllerUSB::writeData:");
    [_writeDataBuffer appendData:data];
    [self writeDataFromBufferToStream];
}


- (BOOL)isAccessoryConnected
{
    //NSLog(@"EAControllerUSB::isAccessoryConnected");
    if (_selectedAccessory && [_selectedAccessory isConnected])
        return YES;
    else
        return NO;
}


#pragma mark -
#pragma mark EAAccessoryDelegate

- (void)accessoryDidDisconnect:(EAAccessory *)accessory
{
    NSLog(@"EAControllerUSB::accessoryDidDisconnect:");
    // do something ...
    //if (![self isAccessoryConnected])
    //{
    
    //}
}


#pragma mark -
#pragma mark NSStreamDelegateEventExtensions

- (void)stream:(NSStream *)aStream handleEvent:(NSStreamEvent)eventCode
{
    //NSLog(@"EAControllerUSB::handleEvent:");
    switch (eventCode) {
        case NSStreamEventNone:
            //NSLog(@"stream %@ event none", aStream);
            break;
        case NSStreamEventOpenCompleted:
            //NSLog(@"stream %@ event open completed", aStream);
            break;
        case NSStreamEventHasBytesAvailable:
            //NSLog(@"stream %@ event bytes available", aStream);
            [self readDataFromStreamToBuffer];
            break;
        case NSStreamEventHasSpaceAvailable:
            //NSLog(@"stream %@ event space available", aStream);
            [self writeDataFromBufferToStream];
            break;
        case NSStreamEventErrorOccurred:
            //NSLog(@"stream %@ event error", aStream);
            break;
        case NSStreamEventEndEncountered:
            //NSLog(@"stream %@ event end encountered", aStream);
            break;
        default:
            break;
    }
}


#pragma mark -
#pragma mark NSNotifications

- (void)accessoryConnected:(NSNotification *)notification
{
    //NSLog(@"EAControllerUSB::accessoryConnected");
    if (![self isAccessoryConnected])
    {
        EAAccessory *a = [self selectedAccessory];
        [self setupControllerForAccessory:a withProtocolString:_protocolString];
        [self closeSession];
        [self openSession];
    }
}


- (void)accessoryDisconnected:(NSNotification *)notification
{
    //NSLog(@"EAControllerUSB::accessoryDisconnected");
    if (![self isAccessoryConnected])
    {
        EAAccessory *a = [self selectedAccessory];
        [self setupControllerForAccessory:a withProtocolString:_protocolString];
        [self closeSession];
        [self openSession];
    }
}


- (EASession *)openSessionForProtocol:(NSString *)protocolString
{
    NSArray *accessories = [[EAAccessoryManager sharedAccessoryManager]connectedAccessories];
    EAAccessory *accessory = nil;
    EASession *session = nil;
    
    for (EAAccessory *obj in accessories)
    {
        if ([[obj protocolStrings] containsObject:protocolString])
        {
            accessory = obj;
            break;
        }
    }
    
    if (accessory)
    {
        session = [[EASession alloc] initWithAccessory:accessory
                                           forProtocol:protocolString];
        if (session)
        {
            [[session inputStream] setDelegate:self];
            [[session inputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop]
                                             forMode:NSDefaultRunLoopMode];
            [[session inputStream] open];
            [[session outputStream] setDelegate:self];
            [[session outputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop]
                                              forMode:NSDefaultRunLoopMode];
            [[session outputStream] open];
            //[session autorelease];
        }
    }
    
    return session;
}



@end
