//
//  BocaBTController.m
//  BocaLibTester
//
//  Created by michael hall on 9/13/16.
//  Copyright © 2016 Boca Systems. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "BocaBTController.h"

@class BocaStoryBoardAppMainViewController;

@implementation EAController


#pragma mark Public Methods

+ (EAController *)sharedController
{
    static EAController *_eaController = nil;
    if (_eaController == nil)
    {
        _eaController = [[EAController alloc] init];
    }
    
    return _eaController;
}



- (id)init
{
    
    //NSLog(@"EAController::init");
    
    //iPads can be connected to mulitiple Boca BlueTooth LE printers at the same time from
    //the main iPad screen using "Settings" -> Bluetooth
    //I tested with 3 Boca BT LE printers and by manipulating the selected BT index value 0 to 2
    //I was able to connect/disconnect with each.  If someone is interested in that approach
    //add something to the main GUI so that the user can indicate which Boca BT printer they
    //want to use when multiples are available.  For now the first printer found (with index 0)
    //will be selected.  Same applies to selectedAccessory function below
    int selectedBTindex = 0;
    
    if (self = [super init])
    {
        //initialization
        _writeDataBuffer = [[NSMutableData alloc] init];
        _accessoryList = [[NSMutableArray alloc] initWithArray:[[EAAccessoryManager sharedAccessoryManager] connectedAccessories]];
        //NSLog(@"accessory count: %lu", (unsigned long)[_accessoryList count]);
        if ([_accessoryList count])
        {
            //display number of connected BT devices on GUI status screen
            //NSString *connectedcountstr = [NSString stringWithFormat:@"%@%d",@"Connected Accessory Count ",[_accessoryList count]];
            //[BocaStoryBoardAppMainViewControllerobject DisplayText:connectedcountstr];
            
            _selectedAccessory = [_accessoryList objectAtIndex:selectedBTindex];
            NSArray *protocolStrings = [_selectedAccessory protocolStrings];
            self.protocolString = [protocolStrings objectAtIndex:0];
            
        }
    }
    return self;
}


- (EAAccessory *)selectedAccessory
{
    int selectedBTindex = 0;
    
    if (_selectedAccessory == nil)
    {
        _accessoryList = [[NSMutableArray alloc] initWithArray:[[EAAccessoryManager sharedAccessoryManager] connectedAccessories]];
        //NSLog(@"accessory count: %lu", (unsigned long)[_accessoryList count]);
        if ([_accessoryList count])
        {
            _selectedAccessory = [_accessoryList objectAtIndex:selectedBTindex];
            NSArray *protocolStrings = [_selectedAccessory protocolStrings];
            self.protocolString = [protocolStrings objectAtIndex:0];
        }
    }
    return _selectedAccessory;
}


- (void)dealloc
{
    [self closeSession];
    [self setupControllerForAccessory:nil withProtocolString:nil];
    
    _writeDataBuffer = nil;
    _accessoryVC = nil;
}


- (void)setupControllerForAccessory:(EAAccessory *)accessory withProtocolString:(NSString *)protocolString
{
    //NSLog(@"EAController::setupControllerForAccessory:");
    _selectedAccessory = [accessory copy];
    _protocolString = [protocolString copy];
}


- (BOOL)openSession
{
    if (_session == nil)
    {
        //NSLog(@"EAController::openSession");
        [_selectedAccessory setDelegate:self];
        
        //_session = [[EASession alloc] initWithAccessory:[self selectedAccessory] forProtocol:@"com.RovingNetworks.btdemo"];
        _session = [[EASession alloc] initWithAccessory:[self selectedAccessory] forProtocol:_protocolString];
        
        if (_session)
        {
            [[_session inputStream] setDelegate:self];
            [[_session inputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
            [[_session inputStream] open];
            
            [[_session outputStream] setDelegate:self];
            [[_session outputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
            [[_session outputStream] open];
            //NSLog(@"opened the session");
            //enable printer function buttons
            
        }
        //else
        //{
        //NSLog(@"creating session failed");
        //}
    }
    
    return (_session != nil);
}


- (void)closeSession
{
    //NSLog(@"EAController::closeSession");
    [[_session inputStream] close];
    [[_session inputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [[_session inputStream] setDelegate:nil];
    [[_session outputStream] close];
    [[_session outputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [[_session outputStream] setDelegate:nil];
    
    //@mdh BT [_session release];
    _session = nil;
    _selectedAccessory = nil;           //Added this for version 2.0  needed to properly end a session so another can be opened
    //NSLog(@"closed the session");
}


- (void)writeData:(NSData *)data
{
    //NSLog(@"EAController::writeData:");
    [_writeDataBuffer appendData:data];
    //@mdh temp test [self writeDataFromBufferToStream];
}


- (BOOL)isAccessoryConnected
{
    //NSLog(@"EAController::isAccessoryConnected");
    if (_selectedAccessory && [_selectedAccessory isConnected])
        return YES;
    else
        return NO;
}


#pragma mark -
#pragma mark EAAccessoryDelegate

- (void)accessoryDidDisconnect:(EAAccessory *)accessory
{
    //NSLog(@"EAController::accessoryDidDisconnect:");
    // do something ...
    //if (![self isAccessoryConnected])
    //{
    
    //}
}


#pragma mark -
#pragma mark NSStreamDelegateEventExtensions

- (void)stream:(NSStream *)aStream handleEvent:(NSStreamEvent)eventCode
{
    //NSLog(@"EAController::handleEvent:");
    switch (eventCode) {
        case NSStreamEventNone:
            //NSLog(@"stream %@ event none", aStream);
            break;
        case NSStreamEventOpenCompleted:
            //NSLog(@"stream %@ event open completed", aStream);
            break;
        case NSStreamEventHasBytesAvailable:
            //NSLog(@"stream %@ event bytes available", aStream);
            //@mdh temp test [self readDataFromStreamToBuffer];
            break;
        case NSStreamEventHasSpaceAvailable:
            //NSLog(@"stream %@ event space available", aStream);
            //@mdh temp test [self writeDataFromBufferToStream];
            break;
        case NSStreamEventErrorOccurred:
            //NSLog(@"stream %@ event error", aStream);
            break;
        case NSStreamEventEndEncountered:
            //NSLog(@"stream %@ event end encountered", aStream);
            break;
        default:
            break;
    }
}


#pragma mark -
#pragma mark NSNotifications

- (void)accessoryConnected:(NSNotification *)notification
{
    //NSLog(@"EAController::accessoryConnected");
    if (![self isAccessoryConnected])
    {
        EAAccessory *a = [self selectedAccessory];
        [self setupControllerForAccessory:a withProtocolString:_protocolString];
        [self closeSession];
        [self openSession];
    }
}


- (void)accessoryDisconnected:(NSNotification *)notification
{
    //NSLog(@"EAController::accessoryDisconnected");
    if (![self isAccessoryConnected])
    {
        EAAccessory *a = [self selectedAccessory];
        [self setupControllerForAccessory:a withProtocolString:_protocolString];
        [self closeSession];
        [self openSession];
    }
}


@end
