///////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  ViewController.h
//  BocaLibTester
//
//  Created by Michael Hall on 10/15/16.
//  Copyright © 2016 Boca Systems. All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////

#import <UIKit/UIKit.h>
#import <MobileCoreServices/MobileCoreServices.h>
#import <UIKit/UIPopoverController.h>
#import <ExternalAccessory/ExternalAccessory.h>
#import <ExternalAccessory/EAAccessory.h>
#import <AssetsLibrary/AssetsLibrary.h>
#import "../../BocaIpadSDK/BocaIpadSDK/BocaIpadSDK.h"

@import Foundation;
@import ExternalAccessory;

@interface ViewController : UIViewController
<UIImagePickerControllerDelegate,
UINavigationControllerDelegate,
UIDocumentInteractionControllerDelegate,
UIPopoverControllerDelegate,
UIPickerViewDataSource,
UIPickerViewDelegate>
{
    IBOutlet UIWebView *webView;
    UIPopoverController *popoverController;
    BOOL newMedia;
}

extern NSString *EADSessionDataReceivedNotification;


//property for assorted buttons
@property (weak, nonatomic) IBOutlet UISegmentedControl *pathButton;                    //Path 1, 2, 3 or 4
@property (weak, nonatomic) IBOutlet UISegmentedControl *resolutionButton;              //200, 300 or 600 DPI
@property (weak, nonatomic) IBOutlet UISegmentedControl *orientationButton;             //Portrait or landscape
@property (weak, nonatomic) IBOutlet UIStepper *sizeButton;                             //Identify stock size used
@property (weak, nonatomic) IBOutlet UISegmentedControl *scaleButton;                   //Scale to ticket size image or not
@property (weak, nonatomic) IBOutlet UISegmentedControl *ditherButton;                  //Dither image or not
@property (weak, nonatomic) IBOutlet UIButton *WiFiButton;                              //connect/disconnect via Wi-Fi
@property (weak, nonatomic) IBOutlet UIButton *BlueToothButton;                         //connect/disconnect via Bluetooth
@property (strong, nonatomic) IBOutlet UIButton *DisplayBTButton;                           //clear display and message areas
@property (weak, nonatomic) IBOutlet UIButton *USBButton;                               //connect/disconnect via USB/Lightning
@property (weak, nonatomic) IBOutlet UIBarButtonItem *SendStringButton;                 //Send text and FGL commands
@property (strong, nonatomic) IBOutlet UIBarButtonItem *statusButton;                   //Read Status
@property (strong, nonatomic) IBOutlet UIBarButtonItem *SelectFileButton;               //Select a PDF or raw TXT file
@property (strong, nonatomic) IBOutlet UIBarButtonItem *TransmitFileButton;             //Transmit PDF or raw TXT file
@property (strong, nonatomic) IBOutlet UIBarButtonItem *SelectImageButton;              //Select Image
@property (strong, nonatomic) IBOutlet UIBarButtonItem *LoadLogoButton;                 //BMP Stored in Printer
@property (strong, nonatomic) IBOutlet UIBarButtonItem *DriverPrintButton;              //Air Print emulation
@property (strong, nonatomic) IBOutlet UIButton *clearButton;                           //clear display and message areas
@property (strong, nonatomic) IBOutlet UIButton *clearUserSpace;                           //clear user space in printer memory
@property (strong, nonatomic) IBOutlet UIButton *cutprintButton;                        //Issue form feed and use cutter
@property (strong, nonatomic) IBOutlet UIButton *nocutprintButton;                      //Issue form feed without cutter

//property for label and text field
@property (strong, nonatomic) IBOutlet UILabel *ConnectionStatusLabel;                  //Displays Wi-Fi/BT connection status
@property (weak, nonatomic) IBOutlet UITextField *fieldFGL;                             //Enter FGL/Text string
@property (weak, nonatomic) IBOutlet UITextField *fieldIP;                              //Enter IP address of printer
@property (weak, nonatomic) IBOutlet UITextField *fieldBT;                              //Enter/Display BT name of printer
@property (weak, nonatomic) IBOutlet UITextView *textOut;                               //used to display return printer status
@property (strong, nonatomic) IBOutlet UITextField *pickerTextField;                    //Display file name selected
@property (weak, nonatomic) IBOutlet UISegmentedControl *helpswitch;                    //Turns On/Off help display
@property (strong, nonatomic) IBOutlet UIPickerView *FilePicker;                        //PDF/Text File Picker
@property (strong, nonatomic) IBOutlet UIImageView *imageView;                          //Displays images
@property (weak, nonatomic) IBOutlet UIScrollView *statOut;
@property (weak, nonatomic) IBOutlet UILabel *sizeOut;
@property (weak, nonatomic) IBOutlet UIWebView *helpview;                               //Help Screen which displays HTML file
@property (weak, nonatomic) IBOutlet UITextField *fieldLogoID;                          //Used to assign an ID number to a Logo
@property (weak, nonatomic) IBOutlet UITextField *fieldTicketCount;                          //Used to control the number of tickets printed.
@property (weak, nonatomic) IBOutlet UITextField *urlPath;

@property (strong, nonatomic) IBOutlet UIButton *printUrlButton;

@property (weak, nonatomic) IBOutlet UITextField *rowValue;                             //Enter Row Value

@property (weak, nonatomic) IBOutlet UITextField *colValue;                             //Enter Col Value

@property (strong, nonatomic) IBOutlet UIButton *setSP
;
//Function definitions for buttons

- (IBAction)PrintUrlClicked:(id)sender;


- (IBAction)WiFiButtonClicked:(id)sender;
- (IBAction)BlueToothButtonClicked:(id)sender;
- (IBAction)USBButtonClicked:(id)sender;
- (IBAction)TransmitString:(id)sender;
- (IBAction)RecData:(id)sender;
- (IBAction)clearStatus:(id)sender;
- (IBAction)ClearUserSpace:(id)sender;
- (IBAction)backgroundTouched:(id)sender;
- (IBAction)HelpSwitch:(id)sender;
- (IBAction)SelectFile: (id)sender;
- (IBAction)TransmitFile:(id)sender;
- (IBAction)SelectImages:(id)sender;
- (IBAction)LoadLogoImage:(id)sender;
- (IBAction)changePath:(id)sender;
- (IBAction)changeResolution:(id)sender;
- (IBAction)changeOrientation:(id)sender;
- (IBAction)changeSize:(id)sender;
- (IBAction)changeScale:(id)sender;
- (IBAction)changeDither:(id)sender;
- (IBAction)DriverPrintImage:(id)sender;
- (IBAction)CutPrint:(id)sender;
- (IBAction)NoCutPrint:(id)sender;
- (IBAction)DisplayBT:(id)sender;
- (IBAction)UrlPrint:(id)sender;
- (IBAction)SetSP:(id)sender;

@end

//Image position and size
const int StartX = 20;
const int StartY = 445;
const int MaxWidth = 728;
const int MaxHeight = 300;


bool WiFi_Status = false;           //WiFi connection status
bool BT_Status = false;             //Bluetooth connection status
bool USB_Status = false;            //USB/Lightning connection status
UIImage *Connect_Icon;              //WiFi/BT connection button icon will be set to connect.png or disconnect.png
NSString *str_size;                 //Used on GUI for display of stock currently selected

//Printer configuration items.  All of these items are controllable from the sample GUI with a call to the SDK
//See methods such as changePath, changeOrientation, etc.
NSString *path;                     //Default to path 1 <P1>
NSString *orientation;              //Default to Landscape mode <LM>
bool mirrored = false;              //Orientation image flipped //@mdh test, not completed
int resolution;                     //Defualt to 300 DPI
double height;                      //Default ticket height default 2.0 inches
double width;                       //Default ticket width default 5.5 inches
int stocksizeindex;                 //integer representing the size index currently selected default is 0 for Concert Stock (2.0 x 5.5)
bool scaled = false;                //Scale to ticket size when printing. Default to actual size, not scaled to ticket size
//Barcodes and text print sharper without dithering.  Color and Grayscale images print much better with dithering while being converted to B&W
bool dithered = true;              //Dither or not when converting a UIImage to monochrome BMP. Default to yes.









