///////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  ViewController.m
//  BocaLibTester
//
//  Created by Michael Hall on 10/15/16.
//  Updated to version 6.0 on 11/30/17
//  Version 6.0 Introduced OpenSessionBTPicker to allow the selection of multiple Boca BT printer
//  Beta Version 6.1 is for lightening to USB - this is not complete
//  Beta Version 6.2 is to improve PDF print quality.  Quality improved, scaling needs work.
//  Beta Version 6.3 6/24/19 Added NULL tests to SDK for CFRelease which may periodically crash when passed a NULL value
//  Copyright © 2019 Boca Systems. All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  This sample code provides two major functions.
//  1. It can be used simply to test a Boca Systems WiFi or Bluetooth to verify printer functionality.
//     Once connected using the send string button will print a simple ticket just to verify the connection
//  2. The code below also demonstrates calls to the SDK library routines should someone want to
//     reuse this code "As Is".  For more details on the routines listed below see them used in the code
//     and read the HTML users guide provided in the application under the HELP button.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Boca Systems SDK Library Routines
//  =================================
//
// Start or end a wi-fi session
//  BOOL OpenSessionWiFi(NSString *ipaddress);
//  void CloseSessionWiFi();
//
// Start or end a Bluetooth session.  OpenSessionBT is used to connect to the only or default BT printer.  OpenSessionBTPicker is used
// when the user has multiple Boca Systems BT printers paired to the iPad and wants to switch from one to the other.
//  BOOL OpenSessionBT();
//  BOOL OpenSessionBTPicker(NSString *BTName);
//  void CloseSessionBT();
//
// Alter printer configuration items from their default values
//  void ChangeConfiguration(NSString *path, int resolution, bool scaled, bool dithered, int stocksizeindex, NSString *orientation);
//
// Send string to the printer.  This can be character data to be printed on the ticket or it can be FGL commands to be executed by the printer
//  void SendString(NSString *string);
//
// Send a file to the printer for printing.  Supported file formats include suffix *.txt, *.bmp, *.png, *.jpg, *.jpeg and *.pdf
// When transmitting a text file the row and column parameters will not be used.  With any of the other file they will be used for positioning
//  bool SendFile(NSURL *filename, int row, int column);
//
// Send an image file to the printer to be stored in user space as a logo.  Supported file formats include suffix *.bmp, *.png, *.jpg, *.jpeg and *.pdf
//  bool DownloadLogo(NSURL *filename, int idnum);
//
// Print a previously downloaded logo
// When printing a logo on a ticket, the row and column parameters will be used for positioning.
//  bool PrintLogo(int idnum, int row, int column);
//
// Clear user space memory
//  void ClearMemory();
//
// Force printer to eject ticket with printed material (data, text and images) including cutting the ticket
//  void PrintCut();
//
// Force printer to eject ticket with printed material (data, text and images) without cutting the ticket
//  void PrintNoCut();
//
// Read status back from printer in the form of a string
//  NSString *ReadPrinter();
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////

#import "ViewController.h"

@interface ViewController ()
@property (nonatomic, strong) NSMutableArray *accessoryList;
@property (nonatomic, strong) EAAccessory *selectedAccessory;
@property (nonatomic, strong) ViewController *eaSessionController;
@property (nonatomic, strong) IBOutlet UIAlertController *protocolSelectionAlertController;
@property (nonatomic, strong) IBOutlet UIView *noExternalAccessoriesPosterView;
@property (nonatomic, strong) IBOutlet UILabel *noExternalAccessoriesLabelView;
@property (nonatomic, strong) NSArray *supportedProtocolsStrings;
// NOTE: EADSessionController is not threadsafe, calling methods from different threads will lead to unpredictable results
//@interface EADSessionController : NSObject <EAAccessoryDelegate, NSStreamDelegate>
//NSObject <EAAccessoryDelegate, NSStreamDelegate>

+(ViewController *)sharedController;

@property (nonatomic, copy) NSString *protocolString;

- (void)setupControllerForAccessory:(EAAccessory *)accessory withProtocolString:(NSString *)protocolString;

@end

@implementation ViewController

@synthesize WiFiButton;
@synthesize BlueToothButton;
@synthesize USBButton;
@synthesize fieldFGL;
@synthesize fieldIP;
@synthesize fieldBT;
@synthesize fieldLogoID;
@synthesize fieldTicketCount;
@synthesize textOut;
@synthesize ConnectionStatusLabel;
@synthesize FilePicker;
@synthesize statOut;
@synthesize scaleButton = _scaleButton;
@synthesize ditherButton = _ditherButton;
@synthesize pathButton = _pathButton;
@synthesize resolutionButton = _resolutionButton;
@synthesize orientationButton = _orientationButton;
@synthesize sizeButton = _sizeButton;
@synthesize imageView;
@synthesize helpview;

@synthesize SendStringButton;
@synthesize statusButton;
@synthesize SelectFileButton;
@synthesize TransmitFileButton;
@synthesize SelectImageButton;
@synthesize LoadLogoButton;
@synthesize DriverPrintButton;

//Image related test fields including SIDE
NSString *FileExtension=@"";
NSString *Testname=@"";
NSURL *URLName;
int idnum;
NSMutableArray *FileArray;
UIImageView *imageView;
UIImage *image;
UIImagePickerController *picker;
NSString *imageName;
NSURL *localUrl;
NSData *myData = nil;                                       //data read in didFinishPickingMediaWithInfo
NSString *const EAAccessoryDidConnectNotification;
//NSString *const EAAccessoryDidDisconnectNotification;
BOOL TestFlag = false;
int TestCount = 0;
NSString *ImageFlag = @"";              //@mdh V6.0

- (void)viewDidLoad
{
    [super viewDidLoad];
    //UIAlertAction *action;
    
    //Initialize both connection buttons with connect icon
    Connect_Icon = [[UIImage imageNamed:@"connect.png"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
    [BlueToothButton setImage:Connect_Icon forState:UIControlStateNormal];
    [WiFiButton setImage:Connect_Icon forState:UIControlStateNormal];
    [USBButton setImage:Connect_Icon forState:UIControlStateNormal];

    //Setting printer configuration default values based upon most common printer and stock
    //Also initialize a few text fields
    [self Initialize_Items];

    //Initialize with printer function buttons off until a connection is made.
    [self Buttons_Off];
    
    //disable the users ability to edit data in status window
    statOut.userInteractionEnabled = NO;                //make the uitextview field for outputing status, readonly
    
    //Setup the scrollable help screen tied to the iPad5.htm file.  By default the web view will be hidden and controlled by UI help switch
    NSURL *htmlFile = [NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"iPad5" ofType:@"htm"]];
    [helpview loadRequest:[NSURLRequest requestWithURL:htmlFile]];
    helpview.hidden = YES;

    //Set up notification to hear about connection and disconnection of external accessory devices
    //[[NSNotificationCenter defaultCenter] addObserver:self
    //                                         selector:@selector(accessoryDidConnect:)
    //                                             name:EAAccessoryDidConnectNotification
    //                                           object:nil];
    //[[NSNotificationCenter defaultCenter] addObserver:self
    //                                         selector:@selector(accessoryDidDisconnect:)
    //                                             name:EAAccessoryDidDisconnectNotification
    //                                           object:nil];
    //[[EAAccessoryManager sharedAccessoryManager] registerForLocalNotifications];

    //[_eaSessionController setupControllerForAccessory:_selectedAccessory withProtocolString:[action title]];
    //_eaSessionController = [ViewController sharedController];
    _accessoryList = [[NSMutableArray alloc] initWithArray:[[EAAccessoryManager sharedAccessoryManager] connectedAccessories]];
    
    if ([_accessoryList count] == 0) {
        [_noExternalAccessoriesPosterView setHidden:NO];
    } else {
        [_noExternalAccessoriesPosterView setHidden:YES];
    }
    // load the UISupportedExternalAccessory property to know which protocolStrings are registered in the app
    NSBundle *mainBundle = [NSBundle mainBundle];
    self.supportedProtocolsStrings = [mainBundle objectForInfoDictionaryKey:@"UISupportedExternalAccessoryProtocols"];
    
    NSURL *appFolder = [[[NSBundle mainBundle] bundleURL] URLByDeletingLastPathComponent];
    NSString *path = appFolder.absoluteString;
    // printf("App Folder: %s\n", [path UTF8String]);
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *docDir = [paths objectAtIndex:0];
    // printf("Docs Directory: %s\n", [docDir UTF8String]);
    //[self DisplayText:@"Documents Directory:"];
    //[self DisplayText:docDir];
    //textOut.selectedRange = NSMakeRange(textOut.text.length - 1, 0);    //implement auto-scroll to bottom of screen
    
    UIPasteboard *pasteBoard = UIPasteboard.generalPasteboard;
    pasteBoard.string = docDir;
}

- (void)accessoryDidConnect:(NSNotification *)notification {

    TestCount++;
    TestFlag = true;
    

    int selectedBTindex = 0;
    
    if (_selectedAccessory == nil)
    {
        _accessoryList = [[NSMutableArray alloc] initWithArray:[[EAAccessoryManager sharedAccessoryManager] connectedAccessories]];
        if ([_accessoryList count])
        {
            _selectedAccessory = [_accessoryList objectAtIndex:selectedBTindex];
            NSArray *protocolStrings = [_selectedAccessory protocolStrings];
            self.protocolString = [protocolStrings objectAtIndex:0];
        }
    }

    
}

- (void)accessoryDidDisconnect:(NSNotification *)notification {
    EAAccessory *disconnectedAccessory = [[notification userInfo] objectForKey:EAAccessoryKey];
    
    TestFlag = false;
    
    //[self DisplayText:@"Accessory Disconnected."];
    if (_selectedAccessory && [disconnectedAccessory connectionID] == [_selectedAccessory connectionID])
    {
        // clear the protocolSelectionAlertController if the accessory is disconnected
        [_protocolSelectionAlertController dismissViewControllerAnimated:YES completion:^(void) {
            self->_selectedAccessory = nil;
            self->_protocolSelectionAlertController = nil;
        }];
        
        
    }

}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////
// IBAction methods
///////////////////////////////////////////////////////////////////////////////////////////////////////////

//Establish a Wi-Fi Connection
- (IBAction)WiFiButtonClicked:(id)sender
{
    
    [fieldIP resignFirstResponder];                                 //hide keyboard
    
    //Set up icon as if connection will fail
    Connect_Icon = [[UIImage imageNamed:@"connect.png"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
    ConnectionStatusLabel.text = @"Not Connected";
    
    //If WiFi status is false, that indicates the channel is currently closed, so open it.
    if(!WiFi_Status)
    {
        //Call SDK Library Routine to Open WiFi channel base on IP address entered
        WiFi_Status = OpenSessionWiFi(fieldIP.text);
        if (WiFi_Status)
        {
            //Connection succeeded so change icon
            Connect_Icon = [[UIImage imageNamed:@"disconnect.png"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];

            //disable Blue Tooth & USB buttons while using Wi-Fi
            BlueToothButton.enabled = false;
            USBButton.enabled = false;          //@mdh 6.0
            fieldIP.enabled = false;            //@mdh 6.0 don't allow change of IP address while connected

            //Display connection status on screen
            ConnectionStatusLabel.text = @"Connected via Wi-Fi";
            
            //Enable printer function buttons on now that a Wi-Fi connection is made
            [self Buttons_On];
        }
        else    //if opening the WiFi channel fails, display message.
        {
            //Perhaps wrong IP address was used, or the printer is not on, or it on but not properly connected to a LAN
            [self DisplayText:@"Unable to establish Wi-Fi connection.  Check iPad settings for Wi-Fi networks."];
            [self DisplayText:@"Verify the Boca Systems printer is on and Wi-Fi enabled."];
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:@"Unable to connect to printer." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
        }
        
    }
    else    //else if the WiFi status is true that indicates the channel is open, so close it.
    {
        //Call SDK Library Routine to close WiFi channel
        CloseSessionWiFi();
        BlueToothButton.enabled = true;                             //enable BT button
        USBButton.enabled = true;                                   //@mdh 6.0
        WiFi_Status = false;
        fieldIP.enabled = true;            //@mdh 6.0 allow change of IP address once disconnected
        //Turn printer function buttons off until a connection is re-established.
        [self Buttons_Off];
        

    }
    
    //Display icon according to connection status
    [WiFiButton setImage:Connect_Icon forState:UIControlStateNormal];
}

- (IBAction)PrintUrlClicked:(id)sender {
}

//This routine was an experiment that did not completely work out.  I was experimenting with three Boca BT printers.
//It is possible for the iPad to be paired with multiple BT printers at the same time
//I was interested in displaying all three on a pop up list and allow the user to decide which one to pair with and open
//RW channels.  I intended to display the choice in the fieldBT text field.  What would actually display on the
//pop up window was the BT printers listed on the "MY DEVICES" list, not currently connected, but previously paired.
//Any new Boca BT printer listed under "OTHER DEVICES" not currently connected and not previously paired was
//excluded from the pop up list.  For now a user can tap on the BT connect button with nothing entered in the fieldBT
//text box and the application will connect to the default BT printer, whether there is one or three currently paired.
//When multiple printers are currently paired the user can pick which one by entering the BT name of the printer into
//the fieldBT and then tapping on the BT connect button.  A user must disconnect from one to enter name and connect to another.
//Look under Setting -> Bluetooth on the iPad to see the list of BT names currently discovered by the iPad.  Once connected
//the user can switch from one to another as indicated above.
- (IBAction)DisplayBT:(id)sender;
{
    
    //NSPredicate *predicate = [NSPredicate predicateWithFormat:@"self CONTAINS 'Boca'"];
    //[[EAAccessoryManager sharedAccessoryManager] showBluetoothAccessoryPickerWithNameFilter:predicate completion:nil]
    
    [[EAAccessoryManager sharedAccessoryManager] showBluetoothAccessoryPickerWithNameFilter:nil completion:^(NSError *error)
     {
     
         if (error)
         {
             NSLog(@"error :%@", error);
         }
         else
         {
             NSLog(@"paired with ");
         }
     }];
     

}

//Establish a Bluetooth Connection
- (IBAction)BlueToothButtonClicked:(id)sender
{
    
    [fieldBT resignFirstResponder];                                 //hide keyboard

    //Set up icon as if connection will fail
    Connect_Icon = [[UIImage imageNamed:@"connect.png"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
    ConnectionStatusLabel.text = @"Not Connected";

    if (!BT_Status)
    {

        //Call SDK Library Routine to Open Bluetooth channel established on Settings Bluetooth screen
        //Since the iPad can be paired to multiple Boca BT printers simultaneously a user can open a channel
        //to the default printer (first one connected) or a user can specify which printer by name.
        //If the BT Text field is blank, open a channel to the default/only BT printer currently paired to the iPad
        //If a specific BT printer is identified by the case sensitive name (such as Boca123456), then open a channel to that paired printer
        if([fieldBT.text isEqual:@""])
            BT_Status = OpenSessionBT();
        else
            BT_Status = OpenSessionBTPicker(fieldBT.text);
        
        if (BT_Status)
        {
            //Connection succeeded so change icon
            Connect_Icon = [[UIImage imageNamed:@"disconnect.png"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
            //disable WiFi & USB buttons while using Bluetooth
            WiFiButton.enabled = false;
            USBButton.enabled = false;          //@mdh 6.0
            fieldBT.enabled = false;            //@mdh 6.0 don't allow change of BT Name while connected

            //Display connection status on screen
            ConnectionStatusLabel.text = @"Connected via BlueTooth";
            
            //Enable printer function buttons on now that a Bluetooth connection is made
            [self Buttons_On];
        }
        else
        {
            [self DisplayText:@"Unable to establish Bluetooth connection.  Check iPad settings for Bluetooth."];
            [self DisplayText:@"Verify the Boca Systems printer is on and Bluetooth enabled."];
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:@"Unable to connect to printer." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
        }

    }
    else
    {

        //Call SDK Library Routine to close Bluetooth channel
        CloseSessionBT();
        WiFiButton.enabled = true;                                  //enable WiFi button
        USBButton.enabled = true;                                   //@mdh 6.0
        BT_Status = false;
        fieldBT.enabled = true;            //@mdh 6.0 allow change of BT Name once disconnected

        //Turn printer function buttons off until a connection is re-established.
        [self Buttons_Off];
    }
    [BlueToothButton setImage:Connect_Icon forState:UIControlStateNormal];
}

/*
//Added USB/Lightning function in version 6.0
- (IBAction)USBButtonClicked:(id)sender
{
    
    //Set up icon as if connection will fail
    Connect_Icon = [[UIImage imageNamed:@"connect.png"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
    ConnectionStatusLabel.text = @"Not Connected";

    //If USB status is false, that indicates the channel is currently closed, so open it.
    if(!USB_Status)
    {
        //Call SDK Library Routine to Open USB channel
        USB_Status = OpenSessionUSB();
        if (USB_Status)
        {
            //Connection succeeded so change icon
            Connect_Icon = [[UIImage imageNamed:@"disconnect.png"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
            //disable Blue Tooth & Wi-Fi buttons while using USB
            BlueToothButton.enabled = false;
            WiFiButton.enabled = false;
            //Display connection status on screen
            ConnectionStatusLabel.text = @"Connected via USB";
            
            //Enable printer function buttons on now that a USB connection is made
            [self Buttons_On];
        }
        else    //if opening the USB channel fails, display message.
        {
            //Perhaps printer is not on, or it on but not properly connected to Lightning port
            [self DisplayText:@"Unable to establish USB connection.  Check iPad settings for USB networks."];
            [self DisplayText:@"Verify the Boca Systems printer is on and USB enabled."];
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:@"Unable to connect to printer." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
        }
        
    }
    else    //else if the USB status is true that indicates the channel is open, so close it.
    {
        //Call SDK Library Routine to close WiFi channel
        CloseSessionUSB();
        BlueToothButton.enabled = true;                             //enable BT button
        WiFiButton.enabled = true;                                  //enable Wi-Fi button
        USB_Status = false;
        //Turn printer function buttons off until a connection is re-established.
        [self Buttons_Off];
        
        
    }
    
    //Display icon according to connection status
    [USBButton setImage:Connect_Icon forState:UIControlStateNormal];
}
*/

//Send raw FGL text commands and strings to the printer
//Example ==> <RC100,100><F11>This is a test<p>
- (IBAction)TransmitString:(id)sender
{
    int x=0;
    int count = [fieldTicketCount.text intValue];

    [fieldFGL resignFirstResponder];                            //hide keyboard
    
    //Call SDK Library Routine to transmit a string to the printer through the open WiFi or BT channel
    for(x=0; x<count; x++)
        SendString(self.fieldFGL.text);
}

//Read returning status (such as Ticket Ack, Out of Stock, X-On, X-Off, etc.) from the printer.
- (IBAction)RecData:(id)sender
{
    NSString *StatusString;
    
    //Call SDK Library Routine to read status from the printer on current open channel, Wi-Fi or Bluetooth
    StatusString = ReadPrinter();
    [self DisplayText:StatusString];
    textOut.selectedRange = NSMakeRange(textOut.text.length - 1, 0);    //implement auto-scroll to bottom of screen

}


#pragma mark - Actions
//while the keyboard is displayed for text entry, it can be dismissed by touching the "Done"
-(IBAction)textfieldReturn:(id)sender
{
    [sender resignFirstResponder];
}

//Turn the HTML webviewer visibility ON/OFF
-(IBAction)HelpSwitch:(id)sender
{
    if (_helpswitch.selectedSegmentIndex > 0)
        helpview.hidden = NO;           //display html help screen
    else
        helpview.hidden = YES;          //hide html help screen
}

//while the keyboard is displayed for text entry, it can be also dismissed just by touching the background of the main screen
-(IBAction)backgroundTouched:(id)sender
{
    [fieldFGL resignFirstResponder];
    [fieldIP resignFirstResponder];
    [fieldBT resignFirstResponder];
    [fieldTicketCount resignFirstResponder];
    [fieldLogoID resignFirstResponder];
}

//When the clear status window button is pressed erase all text on that window, clear any image displayed and file text field
-(IBAction)clearStatus:(id)sender
{
    textOut.text = @"";
    ImageFlag = @"";                    //@mdh V6.0
    [self.imageView setImage:NULL];
    _pickerTextField.text = @"";
    //Restore viewer's original position and size
    self.imageView.frame = CGRectMake(StartX,StartY,MaxWidth,MaxHeight);
}

- (IBAction)SelectFile: (id)sender
{
    
    MoveFiles();
    [self listFilesFromDocumentsFolder];

}

// This IBAction method from "Send File" button is identical to PrintImage below
- (IBAction)TransmitFile: (id)sender
{
    bool status= false;
    int count = [fieldTicketCount.text intValue];
    int x=0;

    //Call SDK library routine SendFile to send to the printer PNG, JPG, JPEG, BMP and PDF document once converted to monochrome BMP.
    //The row,column parameters are used to position the image files.
    //This routine can also be used to send TXT files to the printer.  With a TXT file the call to PrintCut below may not
    //be necessary if an FGL formfeed command <p> or <q> is included in the TXT file.  Also with the TXT file, the row,column
    //parmeters are ignored in the SDK.  In a TXT file the FGL commands <RC#,#> and <SP#,#> are used for positioning of text and graphics
    for(x=0; x<count; x++)
    {
        status = SendFile(URLName, CUR_POS, CUR_POS);

        //Call SDK library routine to issue a form feed command with the cut command
        //PrintCut();
    }
}

// #define TEST_SEND_WEB_IMAGE
// #define TEST_SEND_FILE_IMAGE

//Select images such as BMP, PNG and JPG.  Display image on the screen
- (IBAction)SelectImages:(id)sender
{
    if ([self->popoverController isPopoverVisible])
    {
        [self->popoverController dismissPopoverAnimated:YES];
    }
    else
    {
        //UIImage *newImage = [[UIImage alloc] initWithCGImage: myImage.CGImage scale: 1.0 orientation: UIImageOrientationRight];
        if ([UIImagePickerController isSourceTypeAvailable: UIImagePickerControllerSourceTypeSavedPhotosAlbum])
        {
            UIImagePickerController *imagePicker = [[UIImagePickerController alloc] init];
            imagePicker.delegate = self;
            imagePicker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
            imagePicker.mediaTypes = [NSArray arrayWithObjects:(NSString *) kUTTypeImage,nil];
            imagePicker.allowsEditing = NO;
            
            self->popoverController = [[UIPopoverController alloc]initWithContentViewController:imagePicker];
            
            popoverController.delegate = self;
            
            [self->popoverController presentPopoverFromBarButtonItem:sender
                                            permittedArrowDirections:UIPopoverArrowDirectionUp animated:YES];
            newMedia = NO;
        }
    }
}

//This routine is intended to send, to the printer for storage as a log. The image is a raw monochrome BMP file
//currently being viewed on app image view controller on the iPad.
- (IBAction)LoadLogoImage:(id)sender
{
    bool status= false;
    
    //Call SDK library routine DownloadLogo to send to the printer PNG, JPG, JPEG, BMP and PDF document once converted to monochrome BMP
    //This routine will not print an image, but store it in printer memory, to be printed latter.  If common images are to be printed
    //frequently it will save large amount of time by downloading/storing a logo once, with an ID number.  Then when it is to be
    //printed on a ticket it can be called upon via a logo command including the ID number used to store it.
    idnum = [fieldLogoID.text intValue];
    status = DownloadLogo(URLName, idnum);
    
}

//Call SDK method to clear the user memory space in the printer of previously downloaded data (Logos & Fonts)
- (IBAction)ClearUserSpace:(id)sender
{
    ClearMemory();
}

- (IBAction)CutPrint:(id)sender
{
    //Call SDK method to issue a form feed command with the cut command
    PrintCut();
}

- (IBAction)NoCutPrint:(id)sender
{
    //Call SDK method to issue a form feed command without the cut command
    PrintNoCut();
}

- (IBAction)changePath:(id)sender
{
    switch (_pathButton.selectedSegmentIndex)
    {
        case 0:
            path = @"<P1>";               //Path 1
            break;
            
        case 1:
            path = @"<P2>";               //Path 2
            break;
            
        case 2:
            path = @"<P3>";               //Path 3
            break;
            
        case 3:
            path = @"<P4>";               //Path 4
            break;
            
        default:
            path = @"<P1>";               //Path 1 Default
            break;
    }
    
    //transmit configuration change to the printer using SDK Library routine
    ChangeConfiguration(path, resolution, orientation);
}

- (IBAction)changeResolution:(id)sender
{
    switch (_resolutionButton.selectedSegmentIndex)
    {
        case 0:
            resolution = 1;                   //lg02 - multiplier x1
            break;
            
        case 1:
            resolution = 2;                   //x2
            break;
            
        case 2:
            resolution = 3;                   //x3
            break;
            
        default:
            break;
    }
    
    //transmit configuration change to the printer using SDK Library routine
    ChangeConfiguration(path, resolution, orientation);
}

- (IBAction)changeScale:(id)sender
{
    switch (_scaleButton.selectedSegmentIndex)
    {
        case 0:
            scaled = false;             //Actual Size
            break;
            
        case 1:
            scaled = true;              //Scaled to Ticket size
            break;
            
        default:
            break;
    }
    
    //transmit configuration change to the printer using SDK Library routine
    ChangeConfiguration(path, resolution, orientation);
    [self PrintPreview];                    //@mdh V6.0

}

- (IBAction)changeDither:(id)sender
{
    switch (_ditherButton.selectedSegmentIndex)
    {
        case 0:
            dithered = false;             //No dithering monochrome image
            break;
            
        case 1:
            dithered = true;              //Dither monochrome image
            break;
            
        default:
            break;
    }
    
    //transmit configuration change to the printer using SDK Library routine
    ChangeConfiguration(path, resolution, orientation);
}

- (IBAction)changeOrientation:(id)sender
{
    switch (_orientationButton.selectedSegmentIndex)
    {
        case 0:
            orientation = @"<LM>";        //Landscape mode
            break;
            
        case 1:
            orientation = @"<PM>";        //Portrait mode
            break;
            
        default:
            break;
    }
    
    //call changeSize below to account for orientation change which alters ticket size
    [self changeSize:sender];
    [self PrintPreview];        //@mdh V6.0

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Orientation, stock sizes height and width below, used for scaling images
// In the SDK stock indexes are used to indicate the stocke size.  This value combined with orientation
// makes it clear which stock size is considered in the SDK.  The list below is displayed using
// landscape orientation which is the default
//
// Stock
// Size
// Index    Name/size
// =====    ================================
//  0       Concert (2 x 5.5)
//  1       Cinema (3.25 x 2)
//  2       Credit Card (2.13 x 3.37)
//  3       Receipt (3.25 x 8)
//  4       Ski (3.25 x 6)
//  5       "4 x 6"
//  6       Wristband 1 (11 x 1)
//  7       Wristband 2 (11 x 1.328)
//  8       Letter (8.5 x 11)       does not existed but used for unformatted sizes
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
- (IBAction)changeSize:(id)sender
{
    stocksizeindex = (int)_sizeButton.value;
    switch ((int)_sizeButton.value)
    {
        case 0:
            
            //if landscape mode
            if([orientation isEqual: @"<LM>"])
            {
                _sizeOut.text = @"Concert (2 x 5.5)";
                height = 2.0;
                width = 5.5;
            }
            else //portrait mode
            {
                _sizeOut.text = @"Concert (5.5 x 2)";
                height = 5.5;
                width = 2.0;
            }
            break;
            
        case 1:
            
            //if landscape mode
            if([orientation isEqual: @"<LM>"])
            {
                _sizeOut.text = @"Cinema (3.25 x 2)";
                height = 3.25;
                width = 2.0;
            }
            else //portrait mode
            {
                _sizeOut.text = @"Cinema (2 x 3.25)";
                height = 2.0;
                width = 3.25;
            }
            break;
            
        case 2:
            
            //if landscape mode
            if([orientation isEqual: @"<LM>"])
            {
                _sizeOut.text = @"Credit Card (2.13 x 3.37)";
                height = 2.13;
                width = 3.37;
            }
            else //portrait mode
            {
                _sizeOut.text = @"Credit Card (3.37 x 2.13)";
                height = 3.37;
                width = 2.13;
            }
            break;
            
        case 3:
            
            //if landscape mode
            if([orientation isEqual: @"<LM>"])
            {
                _sizeOut.text = @"Receipt (3.25 x 8)";
                height = 3.25;
                width = 8.0;
            }
            else //portrait mode
            {
                _sizeOut.text = @"Receipt (8 x 3.25)";
                height = 8.0;
                width = 3.25;
            }
            break;
            
        case 4:
            
            //if landscape mode
            if([orientation isEqual: @"<LM>"])
            {
                _sizeOut.text = @"Ski (3.25 x 6)";
                height = 3.25;
                width = 6.0;
            }
            else //portrait mode
            {
                _sizeOut.text = @"Ski (6 x 3.25)";
                height = 6.0;
                width = 3.25;
            }
            break;
            
        case 5:
            
            //if landscape mode
            if([orientation isEqual: @"<LM>"])
            {
                _sizeOut.text = @"4 x 6";
                height = 4.0;
                width = 6.0;
            }
            else //portrait mode
            {
                _sizeOut.text = @"6 x 4";
                height = 6.0;
                width = 4.0;
            }
            break;
            
        case 6:
            
            //if landscape mode
            if([orientation isEqual: @"<LM>"])
            {
                _sizeOut.text = @"Wristband 1 (1 x 11)";        //@mdh 6.0
                height = 1.0;
                width = 11.0;
            }
            else //portrait mode
            {
                _sizeOut.text = @"Wristband 1 (11 x 1)";        //@mdh 6.0
                height = 11.0;
                width = 1.0;
                
            }
            break;
            
        case 7:
            
            //if landscape mode
            if([orientation isEqual: @"<LM>"])
            {
                _sizeOut.text = @"Wristband 2 (1.328 x 11)";        //@mdh 6.0
                height = 1.328;
                width = 11.0;
            }
            else //portrait mode
            {
                _sizeOut.text = @"Wristband 2 (11 x 1.328)";        //@mdh 6.0
                height = 11.0;
                width = 1.328;
            }
            break;
            
        case 8:                                 //used for custom unformatted ticket sizes.  8.5 x 11 does not really exist
            
            //if landscape mode
            if([orientation isEqual: @"<LM>"])
            {
                _sizeOut.text = @"Letter (8.5 x 11)";
                height = 8.5;
                width = 11.0;
            }
            else //portrait mode
            {
                _sizeOut.text = @"Letter (11 x 8.5)";
                height = 11.0;
                width = 8.5;
            }
            break;
            
        default:
           break;
    }
    str_size = _sizeOut.text;
    
    //transmit configuration change to the printer using SDK Library routine
    ChangeConfiguration(path, resolution, orientation);
    [self PrintPreview];            //@mdh V6.0

}

//Air Print emulation
//This routine is intended to print the file currently being viewed on app image view controller via AirPrint emulation.  When this
//software was written Boca Systems printers are NOT AirPrint compatable.  Instead by installing an AirPrint emulation tool
//installed on a networked PC (such as printopia, fingerprint, handyprint or one of many other) air print code can be used
//to print any image to any shared networked Boca Systems printer on the LAN from the iPad.
- (IBAction)DriverPrintImage:(id)sender
{
    
    //if file read. Data read in didFinishPickingMediaWithInfo, when user made selection
    if (myData)
    {
        UIPrintInteractionController *printController = [UIPrintInteractionController sharedPrintController];
        
        if(printController && [UIPrintInteractionController canPrintData:myData])
        {
            
            //printController.delegate = self;
            
            UIPrintInfo *printInfo = [UIPrintInfo printInfo];
            printInfo.outputType = UIPrintInfoOutputGeneral;
            printInfo.jobName = [imageName lastPathComponent];
            printInfo.duplex = UIPrintInfoDuplexLongEdge;
            printController.printInfo = printInfo;
            printController.showsPageRange = YES;
            printController.printingItem = myData;
            
            void (^completionHandler)(UIPrintInteractionController *, BOOL, NSError *) = ^(UIPrintInteractionController *printController, BOOL completed, NSError *error)
            {
                if (!completed && error)
                {
                    //NSLog(@"FAILED! due to error in domain %@ with error code %ld", error.domain, (long)error.code);
                }
            };
            
            [printController presentAnimated:YES completionHandler:completionHandler];
            
        }
    }
    else
    {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:@"File Not Found." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
    }
    
    
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////
// User Interface support methods
///////////////////////////////////////////////////////////////////////////////////////////////////////////

//This method will update the message screen with text messages
- (void)DisplayText:(NSString *)data
{
    //append new text with existing text already on text out message window
    NSString *sout = [NSString stringWithFormat:@"%@%@\r\n", [textOut text], data];
    //Re-display all text in message window
    [textOut setText:sout];
}


//Once image has been selected by user close selection screen and display image
- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info
{
    float w,h,ratio;
    
    [self->popoverController dismissPopoverAnimated:true];
    
    NSString *mediaType = [info objectForKey:UIImagePickerControllerMediaType];
    //NSString *mediaType = [info objectForKey:kUTTypeCompositeContent];
    //kUTTypeCompositeContent
    [self dismissViewControllerAnimated:YES completion:nil];
    if ([mediaType isEqualToString:(NSString *)kUTTypeImage])
    {
        
        image = [info objectForKey:@"UIImagePickerControllerOriginalImage"];
        ImageFlag = @"IMAGE";       //@mdh V6.0

        //To display the image on the tablet the size sometimes needs to be altered because it is too big
        //to fit in the space available on the tablet.  If it is necessary to scale the image displayed then
        //maintaining the image ratio is important.  At the time of this release the max width is 728
        //and the max height is 300.
        CGSize x = image.size;
        w = x.width;
        h = x.height;
        
        //if the width is greater than the max space available
        if(w > MaxWidth)
        {
            ratio = w / MaxWidth;       //establish ratio
            w = w / ratio;              //divide width by ratio established to bring it down to max width
            h = h / ratio;              //divide height by the same ration to maintain size ratio
        }
        //Now check if height is still greater than max height
        if(h > MaxHeight)
        {
            ratio = h / MaxHeight;      //establish ratio
            h = h / ratio;              //divide height by ratio established to bring it down to max height
            w = w / ratio;              //divide width by the same ration to maintain size ratio
        }
        //Set image frame size and Display image
        imageView.frame = CGRectMake(StartX,StartY,w,h);
        [self.imageView setImage:image];
        //ImageFlag = @"IMAGE";         //@mdh V6.0
        //[self PrintPreview];
        
        [picker dismissViewControllerAnimated:YES completion:nil];
        
        NSURL *referenceURL = [info objectForKey:UIImagePickerControllerReferenceURL];
        URLName = referenceURL;                             //Save URL Name for possible calls to SDK routines
        FileExtension = referenceURL.pathExtension;         //Save file extension
        Testname = referenceURL.absoluteString;             //Save string name for the display
        _pickerTextField.text = Testname;
        ALAssetsLibrary *library = [[ALAssetsLibrary alloc] init];
        [library assetForURL:referenceURL resultBlock:^(ALAsset *asset)
         {
             
             ALAssetRepresentation *representation = [asset defaultRepresentation];
             
             uint8_t *buffer = (uint8_t *)malloc(sizeof(uint8_t)*(unsigned long)[representation size]);
             if (buffer != NULL)
             {
                 NSError *error = nil;
                 NSUInteger bytesRead = [representation getBytes:buffer fromOffset:0 length:(NSUInteger)[representation size] error:&error];
                 myData = [NSData dataWithBytes:buffer length:bytesRead];            //read in image file
                 free(buffer);
             }
             
         }failureBlock:nil];
    }
    
}

//Display UIImage created from photo image, on tablet
- (void) DisplayImage
{
    UIImage *newImage;
    
    //To display the image on the tablet the size sometimes needs to be altered because it is too big
    //to fit in the space available on the tablet.  If it is necessary to scale the image displayed then
    //maintaining the image ratio is important.  At the time of this release the max width is 728
    //and the max height is 300.
    CGFloat w,h,ratio,scale;
    
    CGSize x = image.size;
    w = x.width;
    h = x.height;
    
    //@mdh test
    if([orientation isEqual:@"<LM>"])
        scale = width/height;
    else
        scale = height/width;
    
    w = width * resolution;              //based upon stock size width
    h = height * resolution;             //based upon stock size height
    
    //if the width is greater than the max space available
    if(w > MaxWidth)
    {
        ratio = w / MaxWidth;       //establish ratio
        w = w / ratio;              //divide width by ratio established to bring it down to max width
        h = h / ratio;              //divide height by the same ration to maintain size ratio
    }
    
    //Now check if height is still greater than max height
    if(h > MaxHeight)
    {
        ratio = h / MaxHeight;      //establish ratio
        h = h / ratio;              //divide height by ratio established to bring it down to max height
        w = w / ratio;              //divide width by the same ration to maintain size ratio
    }
    
    if([orientation isEqual:@"<LM>"])
    {
        newImage = [[UIImage alloc] initWithCGImage: image.CGImage
                                              scale: 1.0
                                        orientation: UIImageOrientationUp];
        
        //Check User Interface setting for Actual image size or Scaled to Stock size
        if(scaled)
        {
            imageView.contentMode = UIViewContentModeScaleAspectFit;        //Scale to fit stock size
        }
        else
        {
            imageView.contentMode = UIViewContentModeLeft;                  //Actual Size left edge
        }
        
    }
    else            //portrait mode @"<PM>"
    {
        newImage = [[UIImage alloc] initWithCGImage: image.CGImage
                                              scale: scale
                                        orientation: UIImageOrientationUp];
        
        //Check User Interface setting for Actual image size or Scaled to Stock size
        if(scaled)
        {
            imageView.contentMode = UIViewContentModeScaleAspectFit;        //Scale to fit stock size
        }
        else
        {
            imageView.contentMode = UIViewContentModeTopLeft;               //Actual Size top left corner
        }
        
    }
    
    
    
    imageView.frame = CGRectMake(StartX,StartY,w,h);
    [self.imageView setImage:newImage];
    
}

//Display UIImage created from PDF file, on tablet
- (void) DisplayPDF
{
    size_t PAGE_NO = 1;
    
    CGPDFDocumentRef pdf = CGPDFDocumentCreateWithURL((CFURLRef)URLName);
    CGPDFPageRef page = CGPDFDocumentGetPage(pdf, PAGE_NO);
    CGRect rect = CGPDFPageGetBoxRect(page, kCGPDFMediaBox);

    UIGraphicsBeginImageContext(rect.size);
    
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetInterpolationQuality(context, kCGInterpolationHigh);        //V6.2
    CGContextTranslateCTM(context, 0.0, rect.size.height);
    CGContextScaleCTM(context, 1.0, -1.0);
    CGContextSaveGState(context);
    CGContextDrawPDFPage(context, page);
    CGContextRestoreGState(context);
    
    UIImage *resultingImage = UIGraphicsGetImageFromCurrentImageContext();
    
    imageView.frame = CGRectMake(StartX,StartY,rect.size.width,rect.size.height);
    [self.imageView setImage:resultingImage];
    UIGraphicsEndImageContext();

    
}

//File Picker methods for PDF and TXT files
void MoveFiles()
{
    //Turn every file inside the directory into an array
    // Note to self: remember to actually put files in the Documents folder. Use the code in the apparopriately marked file
    NSArray *path = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    
    //strings to actually get the directories
    NSString *appFolderPath = [path objectAtIndex:0];
    NSString *inboxAppFolderPath = [appFolderPath stringByAppendingString:@"/Inbox"]; //add ".plist" to the end of the recipe name
    
    //move all files from inbox to documents root
    //get to inbox directory
    NSString *documentsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    
    NSArray *inboxContents = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:[NSString stringWithFormat:inboxAppFolderPath, documentsDirectory] error:nil];
    
    //move all the files over
    for (int i = 0; i != [inboxContents count]; i++) {
        NSString *oldPath = [NSString stringWithFormat:@"%@/%@", inboxAppFolderPath, [inboxContents objectAtIndex:i]];
        NSString *newPath = [NSString stringWithFormat:@"%@/%@", appFolderPath, [inboxContents objectAtIndex:i]];
        [[NSFileManager defaultManager] moveItemAtPath:oldPath toPath:newPath error:nil];
    }
    //end of moving all files
}

//Move, load and display PDF & Text files
- (void) loadFileFromDocumentsFolder
{
    //MoveFiles();
    //Testname
    //---get the path of the Documents folder---
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    //NSString *filePath = [documentsDirectory stringByAppendingPathComponent:filename];
    NSString *filePath = [documentsDirectory stringByAppendingPathComponent:Testname];
    NSURL *fileUrl = [NSURL fileURLWithPath:filePath];
    URLName = fileUrl;
    
    ImageFlag = @"PDF";         //@mdh V6.0
    [self PrintPreview];
}


- (void)listFilesFromDocumentsFolder
{
    //---get the path of the Documents folder---
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    
    NSString *documentsDirectory = [paths objectAtIndex:0];
    
    NSFileManager *manager = [NSFileManager defaultManager];
    NSArray *fileList = [manager contentsOfDirectoryAtPath:documentsDirectory error:nil];
    NSMutableString *filesStr = [NSMutableString stringWithString:@"Files in Documents folder \n"];
    int count = 0;
    
    //build string for testing and count number of files found
    //Do not need Inbox directory in file list
    for (NSString *s in fileList)
    {
        if(![s isEqualToString:@"Inbox"])
        {
            [filesStr appendFormat:@"%@ \n", s];
            count++;
        }
    }
    
    FileArray = [[NSMutableArray alloc]initWithCapacity:count];
    
    //Do not need Inbox directory in stored in FileArray
    for (NSString *s in fileList)
    {
        if(![s isEqualToString:@"Inbox"])
            [FileArray addObject:s];
    }
    
    FilePicker.dataSource = self;
    FilePicker.delegate = self;
    [FilePicker setShowsSelectionIndicator:YES];
    
    [self.pickerTextField setInputView:FilePicker];
    
}

#pragma mark - UIPickerView DataSource Method

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    //one column
    return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    //variable number of rows
    return [FileArray count];
}

#pragma mark - UIPickerView Delegate Method
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    //populate each row
    return [FileArray objectAtIndex:row];
}

- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
    //once selected keep filename in Testname
    Testname = [FileArray objectAtIndex:row];
    FileExtension = Testname.pathExtension;
    _pickerTextField.text = Testname;
    [self loadFileFromDocumentsFolder];

}

- (void) Buttons_On
{
    //Enable all function buttons once a Bluetooth or Wi-Fi connection is made
    SendStringButton.enabled = true;
    statusButton.enabled = true;
    DriverPrintButton.enabled = true;
    TransmitFileButton.enabled = true;
    LoadLogoButton.enabled = true;

}

- (void) Buttons_Off
{
    //disable all function buttons until a connection is made.
    SendStringButton.enabled = false;
    statusButton.enabled = false;
    DriverPrintButton.enabled = false;
    TransmitFileButton.enabled = false;
    LoadLogoButton.enabled = false;

}

- (void) Initialize_Items
{
    
    //Setting printer configuration default values based upon most common printer and stock
    path = @"<P1>";                     //Default to path 1
    orientation = @"<LM>";              //Default to Landscape mode
    resolution = 300;                   //Defualt to 300 DPI
    height = 2.0;                       //Default to Concert Stock height
    width = 5.5;                        //Default to Concert Stock width
    stocksizeindex = 0;                 //Default stock size index for concert ticket
    str_size = @"Concert (2 x 5.5)";    //set display to concert stock
    ImageFlag = @"";                    //Clear image flag type from PDF or IMAGE   //@mdh V6.0
    
    //Initialize String Fields
    fieldFGL.text = @"<RC100,100><F12>This is a test<p>";       //lg01 - FGL command string to print a quick ticket
    //fieldIP.text = @"10.0.3.49";
    //fieldFGL.text = @"<SP25,50><HW4,4>";
    // fieldFGL.text = @"<RC100,150><F11>https://bocasystems.com/images/larsonComicPicJPG.png<SP25,50><HW4,4>";
    // @"<RC100,100><F11>This is a test.<p>";       //FGL command string to print a quick ticket
    fieldIP.text = @"";
    fieldBT.text = @"";
    fieldLogoID.text = @"1";
    fieldTicketCount.text = @"1";       //@mdh 6.0 control number of tickets printed

    
}

//@mdh V6.0
- (void) PrintPreview
{
    if([ImageFlag isEqualToString:@"PDF"])
        [self DisplayPDF];
    if([ImageFlag isEqualToString:@"IMAGE"])
        [self DisplayImage];      
}

- (IBAction)SendUrlPath:(id)sender {
    
    NSString *UrlPathname = self.urlPath.text;
    
    if (0 == UrlPathname.length)
    {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:@"URL Path is NULL" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        return;
    }
    NSString *dispPath = @"";
    if (!SendWebImage(UrlPathname, CUR_POS, CUR_POS))
    {
        dispPath = [NSString stringWithFormat:@"Cannot Load: %@", UrlPathname];
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:dispPath delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        return;
    }
    /*
    NSURL *url = [NSURL URLWithString:UrlPathname];
    if (url && url.scheme && url.host)
    {
        //dispPath = [NSString stringWithFormat:@"Button Pressed\n\rURL: %@\n\rscheme: %@\n\rhost: %@", url, url.scheme, url.host];
        NSURL *url = [NSURL URLWithString: UrlPathname];
        NSData *urlData = [NSData dataWithContentsOfURL:url];
        if (urlData)
        {
            UIImage *image = [UIImage imageWithData:urlData];
            if (image)
            {
                SendFileImage(image, CUR_POS, CUR_POS);
            }
            else
            {
                dispPath = [NSString stringWithFormat:@"UIImage is NULL: %@", UrlPathname];
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:dispPath delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                [alert show];
                return;
            }
        }
        else
        {
            dispPath = [NSString stringWithFormat:@"Error, Could not load %@", UrlPathname];
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:dispPath delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
            return;
        }
    }
    else
    {
        if (![[NSFileManager defaultManager] fileExistsAtPath:UrlPathname])
        {
            dispPath = [NSString stringWithFormat:@"File does NOT exist: %@", UrlPathname];
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:dispPath delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
            return;
        }
        if (![[NSFileManager defaultManager] isReadableFileAtPath:UrlPathname])
        {
            dispPath = [NSString stringWithFormat:@"File is NOT readable: %@", UrlPathname];
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:dispPath delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
            return;
        }
        NSString *fileExt=[UrlPathname pathExtension];
        NSString *fileBaseName=[[UrlPathname lastPathComponent]stringByDeletingPathExtension];
        NSString *fileDir=[UrlPathname stringByDeletingLastPathComponent];
        NSString *filePath=[NSBundle pathForResource:fileBaseName ofType:fileExt inDirectory:fileDir];
        //printf("fileExt = %s\n", [fileExt UTF8String]);
        //printf("fileBaseName = %s\n", [fileBaseName UTF8String]);
        //printf("fileDir = %s\n", [fileDir UTF8String]);
        //printf("filePath = %s\n", [filePath UTF8String]);
        if (NULL == filePath)
        {
            printf("Error, filePath is NULL\n");
            return;
        }
        UIImage *theImage = [UIImage imageWithContentsOfFile:UrlPathname];
        SendFileImage(theImage, CUR_POS, CUR_POS);
    }
    */
}

- (IBAction)SetSP:(id)sender
{
    int row = [self.rowValue.text intValue];
    int col = [self.colValue.text intValue];
    // printf("Set SP: %d, %d\n", row, col);
    SetStartPosition(row, col);
    NSString * command = [NSString stringWithFormat:@"<SP%d,%d>", row, col];
    SendString(command);
}

@end
